==> 1.3.1.zip: books-without-borders/src/main/java/net/knarcraft/bookswithoutborders/utility/EncryptionHelper.java <==
package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.encryption.GenenCrypt;
import net.knarcraft.bookswithoutborders.encryption.SubstitutionCipher;
import net.knarcraft.bookswithoutborders.state.EncryptionStyle;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.cleanString;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.fixName;

/**
 * Helper class for book encryption
 */
public final class EncryptionHelper {

    private EncryptionHelper() {
    }

    /**
     * Transforms a string key/password into its numerical values
     *
     * @param key <p>The key to transform</p>
     * @return <p>The numbers representing the key's characters</p>
     */
    public static String getNumberKeyFromStringKey(String key) {
        StringBuilder integerKey = new StringBuilder();
        for (int x = 0; x < key.length(); x++) {
            integerKey.append(Character.getNumericValue(Character.codePointAt(key, x)));
        }
        return integerKey.toString();
    }

    /**
     * Encrypts the pages of a book
     *
     * @param book       <p>The book to encrypt</p>
     * @param style      <p>The encryption style to use</p>
     * @param integerKey <p>The encryption key to use</p>
     * @param player     <p>The player trying to encrypt a book</p>
     * @return <p>The pages of the book in encrypted form</p>
     */
    public static List<String> encryptBookPages(BookMeta book, EncryptionStyle style, String integerKey, Player player) {
        List<String> encryptedPages = new ArrayList<>();
        //Scramble the book's contents
        if (style == EncryptionStyle.DNA) {
            //Encrypt the pages using gene-based encryption
            GenenCrypt gc = new GenenCrypt(integerKey);
            for (int x = 0; x < book.getPages().size(); x++) {
                encryptedPages.add(gc.encrypt(book.getPage(x + 1)));
            }
            return encryptedPages;
        } else if (style == EncryptionStyle.SUBSTITUTION) {
            //Encrypt the pages using a substitution cipher
            SubstitutionCipher sc = new SubstitutionCipher();
            for (int x = 0; x < book.getPages().size(); x++) {
                encryptedPages.add(sc.encrypt(book.getPage(x + 1), integerKey));
            }
            return encryptedPages;
        } else {
            BooksWithoutBorders.sendErrorMessage(player, "Invalid encryption style encountered!");
            return null;
        }
    }

    /**
     * Encrypts a book
     *
     * @param player   <p>The player encrypting the book</p>
     * @param mainHand <p>Whether the player is holding the book in its main hand</p>
     * @param key      <p>The key/password to use for encryption</p>
     * @param style    <p>The encryption style to use</p>
     * @return <p>An encrypted version of the book</p>
     */
    public static ItemStack encryptBook(Player player, boolean mainHand, String key, EncryptionStyle style) {
        return encryptBook(player, mainHand, key, style, "");
    }

    /**
     * Encrypts a book
     *
     * @param player    <p>The player encrypting the book</p>
     * @param mainHand  <p>Whether the player is holding the book in its main hand</p>
     * @param key       <p>The key/password to use for encryption</p>
     * @param style     <p>The encryption style to use</p>
     * @param groupName <p>The name of the group to encrypt for, or "" otherwise</p>
     * @return <p>An encrypted version of the book</p>
     */
    public static ItemStack encryptBook(Player player, boolean mainHand, String key, EncryptionStyle style, String groupName) {
        //converts user supplied key into integer form
        String integerKey = EncryptionHelper.getNumberKeyFromStringKey(key);

        BookMeta book = InventoryHelper.getHeldBookMetadata(player, mainHand);

        if (!book.hasPages()) {
            BooksWithoutBorders.sendErrorMessage(player, "Book is empty!");
            return null;
        }

        //Save the book's un-encrypted contents to a file
        BookMeta newMetadata = saveBookPlaintext(groupName, player, book, integerKey);
        if (newMetadata == null) {
            return null;
        }

        //Get the encrypted pages
        List<String> encryptedPages = EncryptionHelper.encryptBookPages(book, style, integerKey, player);
        if (encryptedPages == null) {
            return null;
        }

        //Format the last page just in case
        BookFormatter.formatLastPage(encryptedPages);
        //Remove empty pages
        List<String> newPages = InputCleaningHelper.cleanList(encryptedPages);

        ItemStack encryptedBook = createEncryptedBook(book, newPages, player, newMetadata);

        BooksWithoutBorders.sendSuccessMessage(player, "Book encrypted!");
        return encryptedBook;
    }

    /**
     * Creates a new encrypted book
     *
     * @param book        <p>The book to encrypt</p>
     * @param newPages    <p>The new encrypted pages</p>
     * @param player      <p>The player encrypting the book</p>
     * @param newMetadata <p>The new metadata of the book</p>
     * @return <p>An encrypted version of the book</p>
     */
    private static ItemStack createEncryptedBook(BookMeta book, List<String> newPages, Player player, BookMeta newMetadata) {
        //Create the encrypted book
        ItemStack encryptedBook = new ItemStack(Material.WRITTEN_BOOK);
        book.setPages(newPages);
        encryptedBook.setItemMeta(book);

        //Update item amount
        encryptedBook.setAmount(InventoryHelper.getHeldBook(player, true).getAmount());
        //Set new item metadata
        encryptedBook.setItemMeta(newMetadata);

        return encryptedBook;
    }

    /**
     * Saves a book's plain text to a file
     *
     * @param groupName  <p>The group who's allowed to decrypt the book, or ""</p>
     * @param player     <p>The player trying to encrypt the book</p>
     * @param book       <p>The book to encrypt</p>
     * @param integerKey <p>The key used to encrypt the book</p>
     * @return <p>The new metadata for the book, or null if it could not be saved</p>
     */
    private static BookMeta saveBookPlaintext(String groupName, Player player, BookMeta book, String integerKey) {
        BookMeta newMetadata = book;
        boolean wasSaved;
        if (groupName.trim().isEmpty()) {
            wasSaved = saveEncryptedBook(player, book, integerKey);
        } else {
            newMetadata = saveEncryptedBookForGroup(player, book, groupName);
            wasSaved = newMetadata != null;
        }
        if (wasSaved) {
            return newMetadata;
        } else {
            return null;
        }
    }

    /**
     * Loads an encrypted book
     *
     * @param player              <p>The player trying to load the book</p>
     * @param key                 <p>The encryption key/password for decryption</p>
     * @param deleteEncryptedFile <p>Whether to delete the plaintext file after decryption is finished</p>
     * @return <p>The loaded book, or null if no book could be loaded</p>
     */
    public static ItemStack loadEncryptedBook(Player player, String key, boolean deleteEncryptedFile) {
        ItemStack heldBook = InventoryHelper.getHeldBook(player, true);
        BookMeta bookMetadata = (BookMeta) heldBook.getItemMeta();
        String path = getBookFolder() + "Encrypted" + getSlash();

        if (bookMetadata == null) {
            return null;
        }

        String fileName = "[" + key + "]" + BookHelper.getBookFile(bookMetadata, player, true);
        fileName = fixName(cleanString(fileName), false);

        File file = new File(path + fileName + ".yml");
        if (!file.isFile()) {
            file = new File(path + fileName + ".txt");

            if (!file.isFile()) {
                BooksWithoutBorders.sendErrorMessage(player, "Incorrect decryption key!");
                return null;
            }
        } else {
            try {
                bookMetadata = BookToFromTextHelper.bookFromFile(file, bookMetadata);
            } catch (Exception e) {
                BooksWithoutBorders.sendErrorMessage(player, "Decryption failed!");
                return null;
            }
        }

        if (deleteEncryptedFile) {
            ConsoleCommandSender consoleSender = BooksWithoutBorders.getConsoleSender();
            try {
                if (!file.delete()) {
                    BooksWithoutBorders.sendErrorMessage(consoleSender, "Book encryption data failed to delete upon decryption!");
                    BooksWithoutBorders.sendErrorMessage(consoleSender, "File location:" + file.getPath());
                }
            } catch (Exception e) {
                BooksWithoutBorders.sendErrorMessage(consoleSender, "Book encryption data failed to delete upon decryption!");
                BooksWithoutBorders.sendErrorMessage(consoleSender, "File location:" + file.getPath());
            }
        }

        ItemStack newBook = new ItemStack(Material.WRITTEN_BOOK);//Book(book.getAuthor(), book.getTitle(), pages, 1, 387);
        newBook.setItemMeta(bookMetadata);
        newBook.setAmount(InventoryHelper.getHeldBook(player, true).getAmount());
        return newBook;
    }

    /**
     * Saves an encrypted book to be decryptable for the given group
     *
     * @param player       <p>The player trying to encrypt the book</p>
     * @param bookMetadata <p>Metadata for the book to encrypt</p>
     * @param groupName    <p>The group which should be able to decrypt the book</p>
     * @return <p>The new encrypted metadata for the book, or null if encryption failed</p>
     */
    private static BookMeta saveEncryptedBookForGroup(Player player, BookMeta bookMetadata, String groupName) {
        String path = getBookFolder() + "Encrypted" + getSlash() + cleanString(groupName) + getSlash();
        File dirTest = new File(path);
        //Creates group dir
        if (!dirTest.exists()) {
            try {
                if (!dirTest.mkdir()) {
                    BooksWithoutBorders.sendErrorMessage(BooksWithoutBorders.getConsoleSender(), "Unable to create encryption group folder!");
                    return null;
                }
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        //Generate file name
        String fileName = BookHelper.getBookFile(bookMetadata, player, true);

        List<String> newLore = new ArrayList<>();
        newLore.add(ChatColor.GRAY + "[" + groupName + " encrypted]");
        if (bookMetadata.hasLore()) {
            List<String> oldLore = bookMetadata.getLore();
            if (oldLore != null) {
                newLore.addAll(oldLore);
            }
        }
        bookMetadata.setLore(newLore);

        //Save file
        File file = (BooksWithoutBordersConfig.getUseYml()) ? new File(path + fileName + ".yml") :
                new File(path + fileName + ".txt");
        if (!file.isFile()) {
            try {
                BookToFromTextHelper.bookToYml(path, fileName, bookMetadata);
            } catch (IOException e) {
                e.printStackTrace();
                BooksWithoutBorders.sendErrorMessage(player, "Group encrypted failed!");
                return null;
            }
        }

        return bookMetadata;
    }

    /**
     * Saves an encrypted book to be decryptable for the given user
     *
     * @param player       <p>The player encrypting the book</p>
     * @param bookMetaData <p>Metadata for the book to encrypt</p>
     * @param key          <p>The key to use for encryption</p>
     * @return <p>The new encrypted metadata for the book, or null if encryption failed</p>
     */
    private static Boolean saveEncryptedBook(Player player, BookMeta bookMetaData, String key) {
        String path = getBookFolder() + "Encrypted" + getSlash();

        String fileName = "[" + key + "]" + BookHelper.getBookFile(bookMetaData, player, true);
        fileName = fixName(cleanString(fileName), false);

        //cancels saving if file is already encrypted
        File file = (BooksWithoutBordersConfig.getUseYml()) ? new File(path + fileName + ".yml") :
                new File(path + fileName + ".txt");
        if (file.isFile()) {
            return true;
        }

        try {
            BookToFromTextHelper.bookToYml(path, fileName, bookMetaData);
        } catch (IOException e) {
            e.printStackTrace();
            BooksWithoutBorders.sendErrorMessage(player, "Encryption failed!");
            return false;
        }
        return true;
    }

}

==> 1.3.5.zip: books-without-borders/src/main/java/net/knarcraft/bookswithoutborders/utility/EncryptionHelper.java <==
package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.encryption.GenenCrypt;
import net.knarcraft.bookswithoutborders.encryption.SubstitutionCipher;
import net.knarcraft.bookswithoutborders.state.EncryptionStyle;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.Material;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.cleanString;
import static net.knarcraft.bookswithoutborders.utility.InputCleaningHelper.fixName;

/**
 * Helper class for book encryption
 */
public final class EncryptionHelper {

    private EncryptionHelper() {
    }

    /**
     * Transforms a string key/password into its numerical values
     *
     * @param key <p>The key to transform</p>
     * @return <p>The numbers representing the key's characters</p>
     */
    public static String getNumberKeyFromStringKey(String key) {
        StringBuilder integerKey = new StringBuilder();
        for (int x = 0; x < key.length(); x++) {
            integerKey.append(Character.getNumericValue(Character.codePointAt(key, x)));
        }
        return integerKey.toString();
    }

    /**
     * Encrypts the pages of a book
     *
     * @param book       <p>The book to encrypt</p>
     * @param style      <p>The encryption style to use</p>
     * @param integerKey <p>The encryption key to use</p>
     * @param player     <p>The player trying to encrypt a book</p>
     * @return <p>The pages of the book in encrypted form</p>
     */
    public static List<String> encryptBookPages(BookMeta book, EncryptionStyle style, String integerKey, Player player) {
        List<String> encryptedPages = new ArrayList<>();
        //Scramble the book's contents
        if (style == EncryptionStyle.DNA) {
            //Encrypt the pages using gene-based encryption
            GenenCrypt gc = new GenenCrypt(integerKey);
            for (int x = 0; x < book.getPages().size(); x++) {
                encryptedPages.add(gc.encrypt(book.getPage(x + 1)));
            }
            return encryptedPages;
        } else if (style == EncryptionStyle.SUBSTITUTION) {
            //Encrypt the pages using a substitution cipher
            SubstitutionCipher sc = new SubstitutionCipher();
            for (int x = 0; x < book.getPages().size(); x++) {
                encryptedPages.add(sc.encrypt(book.getPage(x + 1), integerKey));
            }
            return encryptedPages;
        } else {
            BooksWithoutBorders.sendErrorMessage(player, "Invalid encryption style encountered!");
            return null;
        }
    }

    /**
     * Encrypts a book
     *
     * @param player   <p>The player encrypting the book</p>
     * @param mainHand <p>Whether the player is holding the book in its main hand</p>
     * @param key      <p>The key/password to use for encryption</p>
     * @param style    <p>The encryption style to use</p>
     * @return <p>An encrypted version of the book</p>
     */
    public static ItemStack encryptBook(Player player, boolean mainHand, String key, EncryptionStyle style) {
        return encryptBook(player, mainHand, key, style, "");
    }

    /**
     * Encrypts a book
     *
     * @param player    <p>The player encrypting the book</p>
     * @param mainHand  <p>Whether the player is holding the book in its main hand</p>
     * @param key       <p>The key/password to use for encryption</p>
     * @param style     <p>The encryption style to use</p>
     * @param groupName <p>The name of the group to encrypt for, or "" otherwise</p>
     * @return <p>An encrypted version of the book</p>
     */
    public static ItemStack encryptBook(Player player, boolean mainHand, String key, EncryptionStyle style, String groupName) {
        //converts user supplied key into integer form
        String integerKey = EncryptionHelper.getNumberKeyFromStringKey(key);

        BookMeta book = InventoryHelper.getHeldBookMetadata(player, mainHand);

        if (!book.hasPages()) {
            BooksWithoutBorders.sendErrorMessage(player, "Book is empty!");
            return null;
        }

        //Save the book's un-encrypted contents to a file
        BookMeta newMetadata = saveBookPlaintext(groupName, player, book, integerKey);
        if (newMetadata == null) {
            return null;
        }

        //Get the encrypted pages
        List<String> encryptedPages = EncryptionHelper.encryptBookPages(book, style, integerKey, player);
        if (encryptedPages == null) {
            return null;
        }

        //Format the last page just in case
        BookFormatter.formatLastPage(encryptedPages);
        //Remove empty pages
        List<String> newPages = InputCleaningHelper.cleanList(encryptedPages);

        ItemStack encryptedBook = createEncryptedBook(book, newPages, player, newMetadata);

        BooksWithoutBorders.sendSuccessMessage(player, "Book encrypted!");
        return encryptedBook;
    }

    /**
     * Creates a new encrypted book
     *
     * @param book        <p>The book to encrypt</p>
     * @param newPages    <p>The new encrypted pages</p>
     * @param player      <p>The player encrypting the book</p>
     * @param newMetadata <p>The new metadata of the book</p>
     * @return <p>An encrypted version of the book</p>
     */
    private static ItemStack createEncryptedBook(BookMeta book, List<String> newPages, Player player, BookMeta newMetadata) {
        //Create the encrypted book
        ItemStack encryptedBook = new ItemStack(Material.WRITTEN_BOOK);
        book.setPages(newPages);
        encryptedBook.setItemMeta(book);

        //Update item amount
        encryptedBook.setAmount(InventoryHelper.getHeldBook(player, true).getAmount());
        //Set new item metadata
        encryptedBook.setItemMeta(newMetadata);

        return encryptedBook;
    }

    /**
     * Saves a book's plain text to a file
     *
     * @param groupName  <p>The group who's allowed to decrypt the book, or ""</p>
     * @param player     <p>The player trying to encrypt the book</p>
     * @param book       <p>The book to encrypt</p>
     * @param integerKey <p>The key used to encrypt the book</p>
     * @return <p>The new metadata for the book, or null if it could not be saved</p>
     */
    private static BookMeta saveBookPlaintext(String groupName, Player player, BookMeta book, String integerKey) {
        BookMeta newMetadata = book;
        boolean wasSaved;
        if (groupName.trim().isEmpty()) {
            wasSaved = saveEncryptedBook(player, book, integerKey);
        } else {
            newMetadata = saveEncryptedBookForGroup(player, book, groupName);
            wasSaved = newMetadata != null;
        }
        if (wasSaved) {
            return newMetadata;
        } else {
            return null;
        }
    }

    /**
     * Loads an encrypted book
     *
     * @param player              <p>The player trying to load the book</p>
     * @param key                 <p>The encryption key/password for decryption</p>
     * @param deleteEncryptedFile <p>Whether to delete the plaintext file after decryption is finished</p>
     * @return <p>The loaded book, or null if no book could be loaded</p>
     */
    public static ItemStack loadEncryptedBook(Player player, String key, boolean deleteEncryptedFile) {
        ItemStack heldBook = InventoryHelper.getHeldBook(player, true);
        BookMeta bookMetadata = (BookMeta) heldBook.getItemMeta();
        String path = getBookFolder() + "Encrypted" + getSlash();

        if (bookMetadata == null) {
            return null;
        }

        String fileName = "[" + key + "]" + BookHelper.getBookFile(bookMetadata, player, true);
        fileName = fixName(cleanString(fileName), false);

        File file = new File(path + fileName + ".yml");
        if (!file.isFile()) {
            file = new File(path + fileName + ".txt");

            if (!file.isFile()) {
                BooksWithoutBorders.sendErrorMessage(player, "Incorrect decryption key!");
                return null;
            }
        } else {
            try {
                bookMetadata = BookToFromTextHelper.bookFromFile(file, bookMetadata);
            } catch (Exception e) {
                BooksWithoutBorders.sendErrorMessage(player, "Decryption failed!");
                return null;
            }
        }

        if (deleteEncryptedFile) {
            ConsoleCommandSender consoleSender = BooksWithoutBorders.getConsoleSender();
            try {
                if (!file.delete()) {
                    BooksWithoutBorders.sendErrorMessage(consoleSender, "Book encryption data failed to delete upon decryption!");
                    BooksWithoutBorders.sendErrorMessage(consoleSender, "File location:" + file.getPath());
                }
            } catch (Exception e) {
                BooksWithoutBorders.sendErrorMessage(consoleSender, "Book encryption data failed to delete upon decryption!");
                BooksWithoutBorders.sendErrorMessage(consoleSender, "File location:" + file.getPath());
            }
        }

        ItemStack newBook = new ItemStack(Material.WRITTEN_BOOK);//Book(book.getAuthor(), book.getTitle(), pages, 1, 387);
        newBook.setItemMeta(bookMetadata);
        newBook.setAmount(InventoryHelper.getHeldBook(player, true).getAmount());
        return newBook;
    }

    /**
     * Saves an encrypted book to be decryptable for the given group
     *
     * @param player       <p>The player trying to encrypt the book</p>
     * @param bookMetadata <p>Metadata for the book to encrypt</p>
     * @param groupName    <p>The group which should be able to decrypt the book</p>
     * @return <p>The new encrypted metadata for the book, or null if encryption failed</p>
     */
    private static BookMeta saveEncryptedBookForGroup(Player player, BookMeta bookMetadata, String groupName) {
        String path = getBookFolder() + "Encrypted" + getSlash() + cleanString(groupName) + getSlash();
        File dirTest = new File(path);
        //Creates group dir
        if (!dirTest.exists()) {
            try {
                if (!dirTest.mkdir()) {
                    BooksWithoutBorders.sendErrorMessage(BooksWithoutBorders.getConsoleSender(), "Unable to create encryption group folder!");
                    return null;
                }
            } catch (Exception exception) {
                BooksWithoutBorders.getInstance().getLogger().log(Level.SEVERE, "Unable to save group encrypted book");
                return null;
            }
        }
        //Generate file name
        String fileName = BookHelper.getBookFile(bookMetadata, player, true);

        List<String> newLore = new ArrayList<>();
        newLore.add(ChatColor.GRAY + "[" + groupName + " encrypted]");
        if (bookMetadata.hasLore()) {
            List<String> oldLore = bookMetadata.getLore();
            if (oldLore != null) {
                newLore.addAll(oldLore);
            }
        }
        bookMetadata.setLore(newLore);

        //Save file
        File file = (BooksWithoutBordersConfig.getUseYml()) ? new File(path + fileName + ".yml") :
                new File(path + fileName + ".txt");
        if (!file.isFile()) {
            try {
                BookToFromTextHelper.bookToYml(path, fileName, bookMetadata);
            } catch (IOException exception) {
                BooksWithoutBorders.sendErrorMessage(player, "Group encrypted failed!");
                return null;
            }
        }

        return bookMetadata;
    }

    /**
     * Saves an encrypted book to be decryptable for the given user
     *
     * @param player       <p>The player encrypting the book</p>
     * @param bookMetaData <p>Metadata for the book to encrypt</p>
     * @param key          <p>The key to use for encryption</p>
     * @return <p>The new encrypted metadata for the book, or null if encryption failed</p>
     */
    private static Boolean saveEncryptedBook(Player player, BookMeta bookMetaData, String key) {
        String path = getBookFolder() + "Encrypted" + getSlash();

        String fileName = "[" + key + "]" + BookHelper.getBookFile(bookMetaData, player, true);
        fileName = fixName(cleanString(fileName), false);

        //cancels saving if file is already encrypted
        File file = (BooksWithoutBordersConfig.getUseYml()) ? new File(path + fileName + ".yml") :
                new File(path + fileName + ".txt");
        if (file.isFile()) {
            return true;
        }

        try {
            BookToFromTextHelper.bookToYml(path, fileName, bookMetaData);
        } catch (IOException exception) {
            BooksWithoutBorders.sendErrorMessage(player, "Encryption failed!");
            return false;
        }
        return true;
    }

}
